% Debye Theory for focal distance of a perfect asphere
% vector integral

clear all
close all

dx=0.172;
dy=0.172;
dz=0.172;

A0=1;
E0=[0 A0 0];
lambda = 4e-6;

nExtern=1;
nAsphere=2;

Nx=256;
Ny=256;

Ex = E0(1)*ones(Nx,Ny);
Ey = E0(2)*ones(Nx,Ny);

FEx=fft2(Ex,Nx,Ny)/Nx/Ny;
FEy=fft2(Ey,Nx,Ny)/Nx/Ny;

IX = -ceil((Nx-1)/2):1:floor((Nx-1)/2);
IY = -ceil((Ny-1)/2):1:floor((Ny-1)/2);
R = Nx/4*dx;
xap=64;
yap=64;
ixmin=-xap/2;
ixmax=+xap/2;
iymin=-yap/2;
iymax=+yap/2;

IKX=[0:1:floor((Nx-1)/2) -ceil((Nx-1)/2):1:-1];
IKY=[0:1:floor((Ny-1)/2) -ceil((Ny-1)/2):1:-1];

k0 = 2*pi/lambda;
si = [0 0 1];
ki = nExtern*k0*si;

ux = zeros(xap,yap);
uy = zeros(xap,yap);
uz = zeros(xap,yap);

tan_theta_max_sq=tan(85*pi/180)^2;


for iky = IKY,
    ky = 2*pi/Ny/dy*iky;

    for ikx = IKX,
        kx = 2*pi/Nx/dx*ikx;

        %        if abs(FEx(ikx,iky))==0 && abs(FEy(ikx,iky))==0, continue; end
        %        FE = [FEx(ikx,iky)^(ikx+iky) FEy(ikx,iky)^(ikx+iky)];

        FE = [0 A0];

        kBot_sq=kx^2+ky^2;
        kz_sq=(nAsphere*k0)^2-kBot_sq;
        if kz_sq<=0 || kBot_sq/kz_sq>tan_theta_max_sq, continue; end

        %   FIRST boundary
        kz = sqrt(kz_sq);
        k = [kx ky kz];
        s = k/(nAsphere*k0);
        [surfNorm,status] = asphereNormal(s,nAsphere);
        [tte,ttm,status]  = fresnel(nExtern,ki,nAsphere,k,surfNorm);
        M=[tte 0;0 tte];
        if 0~=kx || 0~=ky,
            [M,status] = transferMatrix(ki,nExtern,k,nAsphere,tte,ttm);
        end
        MFE=(M*FE')';
        E = [MFE transversality(MFE,k)];

        %   SECOND boundary
        surfNorm2 = [0 0 1];
        kz2_sq = (nExtern*k0)^2-kBot_sq;
        if kz2_sq<=0 || kBot_sq/kz2_sq>tan_theta_max_sq, continue; end
        kz2 = sqrt(kz2_sq);
        k2 = [kx ky kz2];
        s2 = k2/(nExtern*k0);
        [tte2,ttm2,status]  = fresnel(nAsphere,k,nExtern,k2,surfNorm);
        M2=[tte2 0;0 tte2];
        if 0~=kx || 0~=ky,
            [M2,status] = transferMatrix(k,nAsphere,k2,nExtern,tte2,ttm2);
        end
        MFE2=(M2*MFE')';
        E2 = [MFE2 transversality(MFE2,k2)];

        sprintf('%d  %d',iky,ikx)
        
        for iy = IY,
            y = dy*iy;
            
            for ix = IX,
                x = dx*ix;

                if ix>ixmax || ix<=ixmin || iy>iymax || iy<=iymin, continue; end
%                if x^2+y^2>R^2, continue; end

                r=[x y 0*dz];

                S = (-1)^(ikx+iky)*sphericWave(nAsphere*k0,r,s);
                S2 = (-1)^(ikx+iky)*sphericWave(nExtern*k0,r,s2);

                ux(ix-ixmin,iy-iymin)=ux(ix-ixmin,iy-iymin)+E2(1)*S*S2;
                uy(ix-ixmin,iy-iymin)=uy(ix-ixmin,iy-iymin)+E2(2)*S*S2;
                uz(ix-ixmin,iy-iymin)=uz(ix-ixmin,iy-iymin)+E2(3)*S*S2;

                u=ux+uy+uz;

            end
        end
        figure(1);
        subplot(221);image(u.*conj(u));
        subplot(222);image(ux.*conj(ux));
        subplot(223);image(uy.*conj(uy));
        subplot(224);image(uz.*conj(uz));
    end
end



ux = -A0*i*nAsphere/lambda * ux;
uy = -A0*i*nAsphere/lambda * uy;
uz = -A0*i*nAsphere/lambda * uz;


