% Debye Theory for focal distance of a perfect asphere
% vector integral
% clear all
% close all
% 
Nx=512;
Ny=512;

A0=1;
FE = [A0 0];
lambda = 4e-6;

nExtern=1;
nAsphere=2;

k0 = 2*pi/lambda;
si = [0 0 1];
ki = nExtern*k0*si;

pixR=4;
nugr=nExtern*k0/2/pi;
dnu=nugr/pixR;
dx=lambda*pixR/nExtern/Nx;
dy=lambda*pixR/nExtern/Ny;

IKX=[0:1:ceil((Nx-1)/2) -floor((Nx-1)/2):1:-1];
IKY=[0:1:ceil((Ny-1)/2) -floor((Ny-1)/2):1:-1];
KX=dnu*IKX;
KY=dnu*IKY;

FEX=zeros(Nx,Ny);
FEY=zeros(Nx,Ny);
FEZ=zeros(Nx,Ny);

tic

for iy = 1:Ny,
    ky = KY(iy);
       
    for ix = 1:Nx,
        kx = KX(ix);

        kBot_sq=kx^2+ky^2;
        kz_sq=(nAsphere*k0)^2-kBot_sq;
        
        %   FIRST boundary
        kz = sqrt(kz_sq);
        k = [kx ky kz];
        s = k/(nAsphere*k0);
        
        if s(3)<0.88, continue; end
        
        [surfNorm,status] = asphereNormal(s,nAsphere);
        [tte,ttm,status]  = fresnel(nExtern,ki,nAsphere,k,surfNorm);
        M=[tte 0;0 tte];
        if 0~=kx || 0~=ky,
            [M,status] = transferMatrix(ki,nExtern,k,nAsphere,tte,ttm);
        end
        E1 = (M*FE')';
        
        %   SECOND boundary
        surfNorm2 = [0 0 1];
        kz2_sq = (nExtern*k0)^2-kBot_sq;
        if kz2_sq<=0, continue; end
        kz2 = sqrt(kz2_sq);
        k2 = [kx ky kz2];
        s2 = k2/(nExtern*k0);
        [tte2,ttm2,status]  = fresnel(nAsphere,k,nExtern,k2,surfNorm);
        M2=[tte2 0;0 tte2];
        if 0~=kx || 0~=ky,
            [M2,status] = transferMatrix(k,nAsphere,k2,nExtern,tte2,ttm2);
        end
        E2=(M2*E1')';
        E2 = [E2 transversality(E2,k2)]/s2(3);
        
        FEX(iy,ix)=E2(1);
        FEY(iy,ix)=E2(2);
        FEZ(iy,ix)=E2(3);
        
    end
        
end

toc

EX=-1000i*nExtern/lambda*A0*fftshift(ifft2(fftshift(FEX/Nx/Ny),Nx,Ny));
EY=-100i*nExtern/lambda*A0*fftshift(ifft2(fftshift(FEY/Nx/Ny),Nx,Ny));
EZ=-500i*nExtern/lambda*A0*fftshift(ifft2(fftshift(FEZ/Nx/Ny),Nx,Ny));
A=(EX.*conj(EX)+EY.*conj(EY)+EZ.*conj(EZ));
 
r=64;
EX_plot=EX(Nx/2-r:Nx/2+r,Ny/2-r:Ny/2+r);
EY_plot=EY(Nx/2-r:Nx/2+r,Ny/2-r:Ny/2+r);
EZ_plot=EZ(Nx/2-r:Nx/2+r,Ny/2-r:Ny/2+r);
A_plot=A(Nx/2-r:Nx/2+r,Ny/2-r:Ny/2+r);

figure(111);

subplot(131);surf(sqrt(EX_plot.*conj(EX_plot)), ...
    'Facecolor','interp','edgecolor','none', ...
    'FaceLighting','phong','AlphaDataMapping','scaled');
axis off; axis equal; view([0,90]);title('E_x amplitude');

subplot(132);surf(sqrt(EY_plot.*conj(EY_plot)), ...
    'Facecolor','interp','edgecolor','none', ...
    'FaceLighting','phong','AlphaDataMapping','scaled');
axis off; axis equal; view([0,90]);title('E_y amplitude');

subplot(133);surf(sqrt(EZ_plot.*conj(EZ_plot)), ...
    'Facecolor','interp','edgecolor','none', ...
    'FaceLighting','phong','AlphaDataMapping','scaled');
axis off; axis equal; view([0,90]);title('E_z amplitude');

figure(222);surf(sqrt(A_plot), ...
    'Facecolor','interp','edgecolor','none', ...
    'FaceLighting','phong','AlphaDataMapping','scaled');
axis off; axis equal; view([0,90]);title('E amplitude');
% figure(2);
% subplot(221);mesh(real(E));axis off; view([0,90]);
% subplot(222);mesh(real(EX));axis off; view([0,90]);
% subplot(223);mesh(real(EY));axis off; view([0,90]);
% subplot(224);mesh(real(EZ));axis off; view([0,90]);



