% Debye Theory for focal distance of a perfect asphere
% vector integral
clear all
close all

Nx=256;
Ny=256;

A0=1;
FE = [0 A0];
lambda = 4e-6;

nExtern=1;
nAsphere=2;

k0 = 2*pi/lambda;
si = [0 0 1];
ki = nExtern*k0*si;

pixR=0.9;
nugr=nExtern*k0/2/pi;
dnu=nugr/pixR;
dx=lambda*pixR/nExtern/Nx;
dy=lambda*pixR/nExtern/Ny;

IKX=[0:1:ceil((Nx-1)/2) -floor((Nx-1)/2):1:-1];
IKY=[0:1:ceil((Ny-1)/2) -floor((Ny-1)/2):1:-1];
KX=dnu*IKX;
KY=dnu*IKY;

FEX=zeros(Nx,Ny);
FEY=zeros(Nx,Ny);
FEZ=zeros(Nx,Ny);

matlabpool 4
matlabpool size

tic

for iy = 1:Ny,
    ky = KY(iy);
    
    xFEX=zeros(1,Nx);
    xFEY=zeros(1,Nx);
    xFEZ=zeros(1,Nx);
       
    parfor ix = 1:drange(Nx)
        kx = KX(ix)

        kBot_sq=kx^2+ky^2;
        kz_sq=(nAsphere*k0)^2-kBot_sq;

        %   FIRST boundary
        kz = sqrt(kz_sq);
        k = [kx ky kz];
        s = k/(nAsphere*k0);
        [surfNorm,status] = asphereNormal(s,nAsphere);
        [tte,ttm,status]  = fresnel(nExtern,ki,nAsphere,k,surfNorm);
        M=[tte 0;0 tte];
        if 0~=kx || 0~=ky,
            [M,status] = transferMatrix(ki,nExtern,k,nAsphere,tte,ttm);
        end
        E1 = (M*FE')';
        
        %   SECOND boundary
        surfNorm2 = [0 0 1];
        kz2_sq = (nExtern*k0)^2-kBot_sq;
        if kz2_sq<=0, continue; end
        kz2 = sqrt(kz2_sq);
        k2 = [kx ky kz2];
        s2 = k2/(nExtern*k0);
        [tte2,ttm2,status]  = fresnel(nAsphere,k,nExtern,k2,surfNorm);
        M2=[tte2 0;0 tte2];
        if 0~=kx || 0~=ky,
            [M2,status] = transferMatrix(k,nAsphere,k2,nExtern,tte2,ttm2);
        end
        E2=(M2*E1')';
        E2 = [E2 transversality(E2,k2)]/s2(3);
        
        xFEX(ix)=E2(1);
        xFEY(ix)=E2(2);
        xFEZ(ix)=E2(3);
        
    end
    
    FEX(:,iy)=xFEX;
    FEY(:,iy)=xFEY;
    FEZ(:,iy)=xFEZ;
        
end

matlabpool close

toc

EX=-1i*nExtern/lambda*A0*fftshift(ifft2(fftshift(FEX),Nx,Ny));
EY=-1i*nExtern/lambda*A0*fftshift(ifft2(fftshift(FEY),Nx,Ny));
EZ=-1i*nExtern/lambda*A0*fftshift(ifft2(fftshift(FEZ),Nx,Ny));

E=EX+EY+EZ;

figure(1);
subplot(221);mesh(E.*conj(E));axis off; view([0,90]);
subplot(222);mesh(EX.*conj(EX));axis off; view([0,90]);
subplot(223);mesh(EY.*conj(EY));axis off; view([0,90]);
subplot(224);mesh(EZ.*conj(EZ));axis off; view([0,90]);



